// Rule.cpp: implementation of the CRule class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ruleparser.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRule::CRule(CRuleEngine* parent) : m_Parent(parent)
{

}

CRule::~CRule()
{
	m_conds.clear();
	m_actions.clear();
}

bool CRule::set_XML( IXMLDOMNodePtr e )
{
	_variant_t v;
	_bstr_t s;
	IXMLDOMNodeListPtr l;
	IXMLDOMElementPtr el;

	//
	// read attributes
	//

	v.vt = VT_BSTR;
	e->attributes->getNamedItem( "name" )->get_nodeValue( &v );

	s = v.bstrVal;
	m_Name = s;

	//
	// read conditions
	//

	l = e->selectNodes( L"conds/cond" );

	l->reset();

	el = l->nextNode(); 

	while ( el )
	{
		CCondition *r = new CCondition;
		r->set_XML ( el );
		m_conds.push_back( *r );

		el = l->nextNode(); 
	}

	//
	// read actions
	//
	l = e->selectNodes( L"actions/*" );

	l->reset();

	el = l->nextNode(); 

	while ( el )
	{
		CAction* a = new CAction;
		a->set_XML( el );
		m_actions.push_back( *a );

		el = l->nextNode(); 
	}

	e.Release();

	return true;
}

////////////////////////////////////////////////
// Function Evaluate
// Input: CQuery& query - query object, that responds 
//						  to questions about world state
// Output: TActionVector: list with generated actions
////////////////////////////////////////////////
TActionVector* CRule::Evaluate( CQuery& query )
{
	TConditionVectorIterator i = m_conds.begin();
	TConditionVectorIterator e = m_conds.end();

#ifdef DEBUG
	cout << m_Name << endl;
#endif

	// 
	// evaluate conditions
	//
	for ( ; i!=e; i++ )
	{
		CCondition& c = *i;
		if ( c.evaluate( query ) == false )
			return NULL;
	}

	TActionVectorIterator ai = m_actions.begin();
	TActionVectorIterator ae = m_actions.end();
	TActionVector* av = new TActionVector;

	//
	// Generate action list
	//
	for ( ; ai != ae; ai++ )
	{
		CAction& action = *ai;
		
		//
		// Get all rules by name
		//
		TRuleVector* rules = m_Parent->GetRulesByName( action.name );
		
		if ( rules->size() == 0  )
		{
			//
			// if there are no rules with this name, 
			// consider the action is elementary and add it to the list
			//
			av->push_back( action );	
		}
		else
		{
			//
			// else evaluate the rules
			//
			TRuleVectorIterator ri = rules->begin();
			TRuleVectorIterator re = rules->end();

			for ( ; ri != re; ri++ )
			{
				CRule& r = *ri;
				TActionVector* avtmp = new TActionVector;
				
				avtmp = r.Evaluate( query );

				if ( avtmp )
				{
					// 
					// if the rule has true executable conditions
					// add its result actions to the list
					// 
					TActionVectorIterator aitmp = avtmp->begin();
					TActionVectorIterator aetmp = avtmp->end();

					for ( ; aitmp != aetmp; aitmp++ )
					{
						CAction& actmp = *aitmp;
						av->push_back( actmp );						
					}

					delete avtmp;
					break;
				}
			}
		}
		delete rules;
	}

	return av;
}
